local loader = require "advtileloader/Loader"
loader.path = "maps/"

local HC = require "hardoncollider"

local hero
local collider
local allSolidTiles

function love.load()
	
	-- load the level and bind to variable map
    map = loader.load("bana.tmx")

    -- load HardonCollider, set callback to on_collide and size of 100
    collider = HC(100, on_collide)

    -- find all the tiles that we can collide with
    allSolidTiles = findSolidTiles(map)

    -- set up the hero object, set him to position 32, 32
    setupHero(32,32)
	
	
	player = {
		x = love.graphics.getWidth() / 2 - 124,
		y = -love.graphics.getHeight() / 2 +64,
		image = love.graphics.newImage("assets/player.png"),
		
		ySpeed = 0,
		jump_height = 300,
		
		maxSpeed = 250,
		xSpeed = 0,
		xAcceleration = 450,
		inAirXAcceleration = 275,

		inAir = false,
		moving = false,
		
	}
	
	-- World parameters
	gravity = 900
	airResistance = 25
	temporaryZero = -love.graphics.getHeight() / 2 -- Used for ground until solid blocks and collisions are in place.
	
end

function findSolidTiles(map)

    local collidable_tiles = {}

    local layer = map.tl["ground"]

    for tileX=1,map.width do
        for tileY=1,map.height do

            local tile

            if layer.tileData[tileY] then
                tile = map.tiles[layer.tileData[tileY][tileX]]
            end

            if tile and tile.properties.solid then
                local ctile = collider:addRectangle((tileX-1)*16,(tileY-1)*16,16,16)
                ctile.type = "tile"
                collider:addToGroup("tiles", ctile)
                collider:setPassive(ctile)
                table.insert(collidable_tiles, ctile)
            end

        end
    end

    return collidable_tiles
end

function setupHero(x,y)

    hero = collider:addRectangle(x,y,16,16)
    hero.speed = 50

end



function love.update(dt)
	
	-- Check if air and if so do physics
	inAir(dt)
	
	player.moving = false
	
	-- Acceleration to the right
	if love.keyboard.isDown("right")
	or love.keyboard.isDown("d") then

		player.moving = true
		
		-- If not at max speed, accelerate.
		if player.xSpeed < player.maxSpeed then
			if player.inAir == false then
				player.xSpeed = player.xSpeed + player.xAcceleration * dt
			else 
				player.xSpeed = player.xSpeed + player.inAirXAcceleration * dt
			end	
		end
	end
	
	-- Acceleration to the left
	if love.keyboard.isDown("left")
	or love.keyboard.isDown("a") then
			
		player.moving = true
		
		if player.xSpeed > -player.maxSpeed then
			if player.inAir == false then
				player.xSpeed = player.xSpeed - player.xAcceleration * dt
			else 
				player.xSpeed = player.xSpeed - player.inAirXAcceleration * dt
			end
		end
	end
	
	-- De-acceleration if no movement and not in air.
	if player.moving == false
	and player.inAir == false then
		if player.xSpeed > 0 then
			player.xSpeed = player.xSpeed - player.xAcceleration * dt
		elseif player.xSpeed < 0 then
			player.xSpeed = player.xSpeed + player.xAcceleration * dt
		end
	end
	
	-- Horizontal movement
	player.x = player.x + player.xSpeed * dt
	
end

-- Other key presses, all key presses should probably be gathered somewhere.
function love.keypressed(key)
	
	-- Quitting
	if key == "escape" then
      love.event.quit()
	end
	
	-- Jumping
	if key == " "
	and player.inAir == false then
		player.ySpeed = player.jump_height
	end
end

-- Jumping, falling and horizontal de-acceleration in air due to air resistance.
function inAir (dt)
		if player.ySpeed ~= 0 then
			player.inAir = true
	end
	
	if player.inAir == true then
		player.y = player.y + player.ySpeed * dt
		player.ySpeed = player.ySpeed - gravity * dt
		
		if player.xSpeed > 0 then
		player.xSpeed = player.xSpeed - airResistance * dt
		elseif player.xSpeed < 0 then
		player.xSpeed = player.xSpeed + airResistance * dt
		end
		
		if player.y < temporaryZero then -- Hit ground
			player.inAir = false
			player.ySpeed = 0
			player.y = temporaryZero
		end
	end
end  

		
function love.draw()
	love.graphics.draw(player.image, player.x, -player.y)
	
	map:draw()
end